/*
* mpu.h                                                     Version 5.1.0
*
* ARM-M MPU definitions.
*
* Copyright (c) 2016-2021 Micro Digital Inc.
* All rights reserved. www.smxrtos.com
*
* SMX is protected by patents listed in smx.h.
*
* This software is confidential and proprietary to Micro Digital Inc.
* It has been furnished under a license and may be used, copied, or
* disclosed only in accordance with the terms of that license and with
* the inclusion of this header. No title to nor ownership of this
* software is hereby transferred.
*
* Author: Ralph Moore
*
*****************************************************************************/

#ifndef SB_MPU_H
#define SB_MPU_H

/* Configuration */

#define MP_MPA_DEV     1  /* 1 during development, 0 for better performance */
/* BR_EN is in XSMX\xarmm_iar.inc */

/* Number of active and static MPU slots */
#define MP_MPU_ACTVSZ  8
#define MP_MPU_STATSZ  0
#define MP_MPU_FAS     MP_MPU_STATSZ
#define MP_MPU_SZ      (MP_MPU_STATSZ + MP_MPU_ACTVSZ)

/* Definitions */

#define MP_DYN_RGN(dyn) {(u32)&dyn, MP_DRT}
#define MP_TMSK_DFLT (0xFFFFFFFF >> (32 - MP_MPU_ACTVSZ))
#define MP_EN        (1 << 0)   /* region enable */
#if SB_CPU_ARMM7
#define MP_DRT       0x80000000 /* dynamic region test for RASR -- bit31 == 1 */
#define MP_DATARW    ((1 << 28) | (3 << 24) | (1 << 17))  /* XN |  RW | C  */
#define MP_DATARO    ((1 << 28) | (6 << 24) | (1 << 17))  /* XN |  RO | C  read only data */
#define MP_CODE      (            (6 << 24) | (1 << 17))  /*       RO | C  code */
#define MP_IOR       ((1 << 28) | (3 << 24))              /* XN | PRW      read/write data non-cacheable */
#define MP_PDATARW   ((1 << 28) | (1 << 24) | (1 << 17))  /* XN | PRW | C  priv read/write data */
#define MP_PDATARO   ((1 << 28) | (5 << 24) | (1 << 17))  /* XN | PRO | C  priv read only data */
#define MP_PCODE     (            (5 << 24) | (1 << 17))  /*      PRO | C  priv code */
#define MP_PIOR      ((1 << 28) | (1 << 24))              /* XN | PRW      priv read/write data non-cacheable */
#define MP_V         (1 << 4)    /* region valid */
#elif SB_CPU_ARMM8
#define MP_DRT       0xFFFFFFFF /* dynamic region test for RLAR -- illegal value */
#define MP_DATARW    ((000<<5) | (00<<3) | (01<<1) | 1) /* AI | SH | AP | XN */
#define MP_DATARO    0x7        /* read only data << 1 | XN */
#define MP_CODE      0x6        /* code */
#define MP_IOR       0x3        /* read/write data */
#define MP_PDATARW   0x1        /* priv read/write data << 1 | XN */
#define MP_PDATARO   0x5        /* priv read only data << 1 | XN */
#define MP_PCODE     0x4        /* priv code */
#define MP_PIOR      0x1        /* priv read/write data */
#endif

#if MP_MPA_DEV
#define MP_MPR_SZ 3     /* words */
#else
#define MP_MPR_SZ 2     /* words */
#endif

#define mp_MPA_PTR(t, n) (u32*)t->mpap + (MP_MPR_SZ)*n

typedef struct {        /* MEMORY PROTECTION REGION */
  #if SB_CPU_ARMM7
   u32  rbar;              /* region address and slot */
   u32  rasr;              /* region attributes and size */
  #elif SB_CPU_ARMM8
   u32  rbar;              /* region address, shareability, and access permissions */
   u32  rlar;              /* region last address, attributes index, and EN */
  #endif
  #if MP_MPA_DEV
   const char *name;       /* region name (for debugging) */
  #endif
} MPR, *MPR_PTR;

typedef MPR MPA[];      /* Memory Protection Array */

extern MPA  mpa_dflt;   /* default MPA */

/* Function Prototypes */

#ifdef __cplusplus
extern "C" {
#endif
void    cp_Init(void);
BOOLEAN mp_RegionSysDataTest(u32* mp);
#ifdef __cplusplus
}
#endif

/* Error Checks */

#if (SMX_CFG_MPU && (MP_MPU_ACTVSZ == 0))
#error MP_MPU_ACTVSZ invalid.
#endif

#endif /* SB_MPU_H */
