/*
* bcc.h                                                     Version 5.2.0
*
* C Compiler Definitions and Run-Time Library Replacements include file.
* The main purposes of this file are to:
*
* 1. define macros to map onto some compiler keywords, for portability.
* 2. include C run-time library header files and definitions that MUST
*    precede smx header files when there is an issue such as the heap
*    functions vs. smx's translation macros.
* 3. supply prototypes for functions missing in some compilers' RTLs.
* 4. include common RTL headers so it's not necessary to add includes
*    to many files.
*
* Note: bdef.h must be included before this file since it defines the
*       target-related constants. smx headers should be included after
*       this file.
*
* Copyright (c) 2003-2024 Micro Digital Inc.
* All rights reserved. www.smxrtos.com
*
* This software is confidential and proprietary to Micro Digital Inc.
* It has been furnished under a license and may be used, copied, or
* disclosed only in accordance with the terms of that license and with
* the inclusion of this header. No title to nor ownership of this
* software is hereby transferred.
*
* Author: David Moore
*
*****************************************************************************/

#ifndef SB_BCC_H
#define SB_BCC_H

/*===========================================================================*
*          C RTL Headers and Defines that Must Precede smx Headers           *
*===========================================================================*/

/*
*  Note: Including stdlib.h ahead of xapi.h, since otherwise, translation
*        macros in xapi.h would cause heap call names in stdlib.h to be
*        translated to smx names, causing build errors.
*/
#include <stdlib.h>


/*===========================================================================*
*                             Other C RTL Headers                            *
*===========================================================================*/
/*
   These do not need to precede smx headers, but are here for convenience.
*/

#include <string.h>     /* memcpy, memset, etc. */
#include <stdarg.h>

#if defined(SB_CPU_ARMM) && (defined(__ICCARM__) || defined(__CROSSWORKS_ARM))
#include <intrinsics.h>
#endif


/*===========================================================================*
*                             Compiler Defines                               *
*===========================================================================*/

#if defined(__IAR_SYSTEMS_ICC__)
#include "ctype.h"
#endif

#ifdef __cplusplus
extern "C" {
#endif

/* packed is to make sure structure members are not 32 bit aligned for some compilers */

#if defined(__CC_ARM)                     /* ARM RealView / Keil */
#define __packed
#define __packed_gnu __attribute__((packed))
#define __packed_pragma 0
#define __short_enum_attr
int _stricmp(const char *__s1, const char *__s2);
int _strnicmp(const char *__s1, const char *__s2, size_t __n);
char * _strupr(char * s);
char * _ultoa(unsigned long v, char * str, int r);
int toupper(int c);
#define __interdecl __irq
#define __interrupt 

#elif defined(__GNUC__)                   /* GNU C/C++ */
#define __inline__      __inline__
#define __interdecl
#define __interrupt __attribute__((interrupt))
#if !defined(__packed)  /* see note <1> */
#define __packed
#endif
#define __packed_gnu    __attribute__((packed))
#if defined(SB_CPU_RX)
#define __packed_pragma 1
#else
#define __packed_pragma 0
#endif
#define __short_enum_attr __attribute__((packed))
#define __unaligned
int _stricmp(const char *__s1, const char *__s2);
int _strnicmp(const char *__s1, const char *__s2, size_t __n);
char * _strupr(char * s);
 #if !defined(__CROSSWORKS_ARM)
char * _ultoa(unsigned long v, char * str, int r);
int toupper(int c);
 #endif

#elif defined(__IAR_SYSTEMS_ICC__)        /* IAR EWARM */
#define __inline__      inline
#define __interdecl
#define __interrupt
#define __packed
#define __packed_gnu
#define __packed_pragma 1
#define __short_enum_attr
#define __unaligned
int _stricmp(const char *__s1, const char *__s2);
int _strnicmp(const char *__s1, const char *__s2, size_t __n);
char * _strupr(char * s);
char * _ultoa(unsigned long v, char * str, int r);

#else
#error Define __packed etc. macros for your compiler in bdef.h.
#define __inline__
#define __interdecl
#define __interrupt
#define __packed
#define __packed_gnu
#define __packed_pragma 1
#define __short_enum_attr
#define __unaligned
#endif

#ifdef __cplusplus
}
#endif

#define SB_PACKED_STRUCT_SUPPORT 1

#define SB_BYTES_TO_CHARS(size)  (size)

#if defined(__GNUC__)
#include "ctype.h"

/* Stringification Macros. See "3.4 Stringification" in GNU C Preprocessor
   manual. These allow using #defined constants/macros in inline assembly,
   such as:
      __asm("mov  r0, #" stringify_expanded(SOME_CONSTANT) "\n\t");
   stringify() makes the name of the constant/macro into a string.
   stringify_expanded() makes the value of the constant/macro into a string.
*/
#define stringify_expanded(x) stringify(x)  /* Expand argument then stringify */
#define stringify(x)          #x            /* Stringify argument */
#endif /* __GNUC__ */


/*===========================================================================*
*                               Other RTL Stuff                              *
*===========================================================================*/

/* itoa(), ltoa(), etc. */

/* These functions are not supplied with some compilers so we define our
   own versions. Here we define prototypes in case the compiler doesn't.
*/

#ifdef __cplusplus
extern "C" {
#endif
char * _itoa(int val, char *str, int radix);
char * _ltoa(long val, char *str, int radix);
char * _ultoa(unsigned long val, char *str, int radix);
char * _strupr(char *str);
#ifdef __cplusplus
}
#endif

#define  itoa   _itoa
#define  ltoa   _ltoa
#define  ultoa  _ultoa
#define  strupr _strupr


/* Configuration: Specify whether to use C library functions (1) or ours (0). */

#define SB_CC_XTOA 0  /* itoa() and ltoa() */

#define SB_CC_ULTOA 0

#if defined(__CC_ARM) || defined(__CROSSWORKS_ARM) || defined(__GNUC__) || \
    defined(__IAR_SYSTEMS_ICC__)
/* ARM RealView / Keil, CrossWorks, GNU C/C++, IAR do not supply _stricmp().
*/
#define SB_CC_STRICMP 0
#else
#define SB_CC_STRICMP 1
#endif

#if defined(__CROSSWORKS_ARM)
/* CrossWorks and some CodeWarrior ColdFire do not supply _strnicmp(). 
*/
#define SB_CC_STRNICMP 0
#else
#define SB_CC_STRNICMP 1
#endif

#if defined(__CC_ARM) || defined(__CROSSWORKS_ARM) || defined(__IAR_SYSTEMS_ICC__)
/* ARM/Keil, CrossWorks, and IAR do not supply _strupr().
*/
#define SB_CC_STRUPR 0
#else
#define SB_CC_STRUPR 1
#endif

#if defined(__GNUC__)
/* GNU C/C++ do not supply toupper().
*/
#define SB_CC_TOUPPER 0
#else
#define SB_CC_TOUPPER 1
#endif

#if defined (__CROSSWORKS_ARM)
#define SB_CC_TIME_FUNCS 0
#else
#define SB_CC_TIME_FUNCS 1
#endif


/*===========================================================================*
*                                  Undefines                                 *
*===========================================================================*/

/* Undefine symbols that conflict with our definitions. Global .h files should
   never #define macros with very short names. The substitutions cause strange
   compiler errors that are difficult to understand.
*/
#if defined(SCB)  /* ARM CMSIS core_*.h defines this */
#undef SCB
#endif


/*
   Notes:
   1. Newer versions of gcc define __packed, but it still must be used at
      the end of a struct definition, not the start as for other compilers.
      We added this check to avoid a warning about redefining it, but the
      compiler will generate warning: '__packed__' attribute ignored for
      all structs that use __packed at the start. (Note that the warning
      is reported for the last line of the struct, making it look like it
      applies to the __packed_gnu macro, which isn't the case.) You could
      turn off the warning or rename our __packed macro and change all uses.
*/

#endif /* SB_BCC_H */

