/*
* xarmm.h                                                   Version 5.2.0
*
* ARM-M (e.g. Cortex-M) definitions and macros, including C scheduler
* macros. The smx Porting Guide documents these macros.
*
* Copyright (c) 2008-2024 Micro Digital Inc.
* All rights reserved. www.smxrtos.com
*
* SMX is protected by patents listed in smx.h.
*
* This software is confidential and proprietary to Micro Digital Inc.
* It has been furnished under a license and may be used, copied, or
* disclosed only in accordance with the terms of that license and with
* the inclusion of this header. No title to nor ownership of this
* software is hereby transferred.
*
* Authors: David Moore, Ralph Moore
*
*****************************************************************************/

#ifndef SMX_XARMM_H
#define SMX_XARMM_H

#define SMX_ISR_C      1  /* ISRs can be fully written in C */

#if SMX_CFG_MPU && SMX_CFG_MPU_ENABLE
/* smx_MPU_BR_ON() turns on the MPU background region */
#define smx_MPU_BR_ON() \
   { \
      *ARMM_MPU_CTRL = 0x5; /* enable MPU with background region */ \
      __DSB(); \
      __ISB(); /* allow background region on to take effect */ \
   }

/* smx_MPU_BR_OFF() turns off the MPU background region */
#define smx_MPU_BR_OFF() \
   { \
      __DMB(); \
      *ARMM_MPU_CTRL = 0x1; /* enable MPU without background region */ \
      __DSB(); \
      __ISB(); /* allow background region off to take effect */ \
   }

/* smx_MPU_BR_SAVE_ON() saves and turns on the MPU background region */
#define smx_MPU_BR_SAVE_ON() \
   { \
      sb_INT_DISABLE(); \
      if ((*ARMM_NVIC_INT_CTRL & ARMM_FL_RETTOBASE) || (*ARMM_NVIC_SYSHAN_CTRL_STATE & 0x7FF)) /* <4> for second check */ \
      { \
         if (*ARMM_MPU_CTRL & 0x4) \
            smx_mpu_br_off = FALSE; \
         else \
            smx_mpu_br_off = TRUE; \
      } \
      smx_MPU_BR_ON(); \
      sb_INT_ENABLE(); \
   }

/* smx_MPU_BR_RESTORE_OFF() turns the MPU background region off if it was off 
   at point of interrupt. It is on from smx_ISR_ENTER(). Used in code below 
   where interrupts are DISABLED(). */
#define smx_MPU_BR_RESTORE_OFF() \
   { \
      if ((*ARMM_NVIC_INT_CTRL & ARMM_FL_RETTOBASE) || (*ARMM_NVIC_SYSHAN_CTRL_STATE & 0x7FF)) /* <4> for second check */ \
      { \
         if (smx_mpu_br_off) \
            smx_MPU_BR_OFF(); \
      } \
   }
#else
#define smx_MPU_BR_ON()
#define smx_MPU_BR_OFF()
#define smx_MPU_BR_SAVE_ON()
#define smx_MPU_BR_RESTORE_OFF()
#endif

/*
   smx_ISR_ENTER() / smx_ISR_EXIT(): Much simpler for ARM-M than ARM so we can
   do them here in C with a little inline assembly.

   IMPORTANT: smx_srnest handling is different from all other ports done so far.
              Because ISR nesting is tracked by the interrupt controller
              (RETTOBASE flag), we do not need to increment/decrement smx_srnest
              in ISRs (except where it goes to the scheduler). This is a big
              deal because it means we don't need to ensure the ISR starts
              with all interrupts disabled. Also, because of PendSV and
              because some registers are automatically pushed onto the task's
              stack, the code here differs substantially from other ports.
*/

#define smx_ISR_ENTER() \
   { \
      smx_SAVE_SUSPLOC_ISR(); \
      smx_RTC_ISR_START() \
      smx_MPU_BR_SAVE_ON();  /* so ISR can access its data regardless of */ \
                             /* which task it interrupts <5> */ \
   }

#define smx_ISR_EXIT()   /* different from other ports -- see comment above */ \
   { \
      smx_RTC_ISR_END() \
      \
      /* if not nested ISR */ \
      sb_INT_DISABLE();  /* make ARMM_NVIC_INT_CTRL read atomic <3> */ \
      if (*ARMM_NVIC_INT_CTRL & ARMM_FL_RETTOBASE) \
      { \
         sb_INT_ENABLE();  /* added with DISABLE above <3> */ \
         /* if LSR or SSR not interrupted */ \
         if (smx_srnest == 0) \
         { \
            sb_INT_DISABLE(); \
            if (smx_lqctr != 0) \
            { \
               smx_srnest = 1; \
               /* Trigger PendSV exception so smx_PreSched (xarmm_xxx.s) will */ \
               /* run LSRs and do task switch if nec. Will run after return */ \
               /* not after sb_INT_ENABLE(), since it has lowest priority. */ \
               *ARMM_NVIC_INT_CTRL = ARMM_FL_PENDSVSET; \
               sb_INT_ENABLE(); \
               return; /* tail-chains to lower priority smx_PreSched (smx_PendSV_Handler) here */ \
            } \
         } \
         smx_RTC_TASK_START(); \
      } \
      /* Need to reenable interrupts because exception exit does NOT. */ \
      /* But we don't want interrupts until after we return, so we */ \
      /* first set FAULTMASK, which IS cleared by the processor on */ \
      /* exception exit. See Note 1 at file end for more information. */ \
      /* This is here not just for the sb_INT_DISABLE() above, but also */ \
      /* in case the ISR used sb_INT_DISABLE() and left interrupts disabled */ \
      /* assuming the exception exit would reenable them, as on other */ \
      /* processors. */ \
      \
      sb_INT_DISABLEF();  /* FAULTMASK = 1. Disable interrupts until exit done. */ \
      sb_INT_ENABLE();    /* BASEPRI = 0 or PRIMASK = 0 */ \
      smx_MPU_BR_RESTORE_OFF();  /* has to be done here with ints disabled */ \
      return;             /* exception exit (to point of interrupt) */ \
   }

#if defined(__CC_ARM)
/*===========================================================================*
*                                  ARM C/C++                                 *
*===========================================================================*/

#error Implement ARM C/C++ case here in xarmm.h.


#elif defined(__GNUC__) || defined(__ICCARM__)
/*===========================================================================*
*                                  GNU C/C++                                 *
*===========================================================================*/
/*
   These are implemented as routines in xarmm_gcc.s. We could not implement
   most inline (here) because of these problems with the GNU compiler:

   1. Use of __asm("LDR r0, =global_var") results in an error that we have
      been unable to fix. Using MOV instead results in a different error.
      We were unable to find a way to simply move a global into a register!

   2. We are told that the GNU compiler will assume inline assembly does
      not change any registers, not even the scratch ones (r0-r3). If the
      compiler were to use the same rule for inline assembly as it does
      for function calls, then there would be no problem using r0-r3 in
      the inline code. Using an advanced syntax of the __asm() statement,
      it might be possible to reserve registers for use in the inline code,
      but it was easier to just code these as assembly routines in xarm_gcc.s.
*/

/*===========================================================================*
*                               IAR ARM C/C++                                *
*===========================================================================*/
/*
   Notes:
   1. Support for inline assembly in the IAR compiler is weak, so we
      simply map these macros onto routines in xarmm_iar.s. Specifically,
      a. It does not support the pseudoinstruction "LDR r0, =global_var".
      b. It does not allow using temporary registers or local variables.
      c. It requires saving any registers used in the macro. By making
         the macros do a function call, we can use the volatile registers
         because the compiler knows not to expect them to be preserved
         across a function call. The compiler does not have a way to
         let us tell it to pick a temporary register we can use, and it
         also can't access autovariables by name.
      IAR's documentation of inline assembly recommends against its use
      and points out it is likely the code could break in new releases
      of the compiler. It suggests that calling assembly routines may
      also have better performance.
*/

/* Scheduler Macros */

/* In xarmm_iar.s */
#ifdef __cplusplus
extern "C" {
#endif
extern void mp_MPULoad_M8(u32* mp);
extern void smx_SwitchToNewStack(void);
extern void smx_SwitchStacks(void);
extern void smx_SwitchToPSP(void);
#ifdef __cplusplus
}
#endif

#define SMX_RSA_SIZE  32  /* register save area size above top of stack */

#define smx_SWITCH_TO_NEW_STACK() smx_SwitchToNewStack();
#define smx_SWITCH_STACKS()       smx_SwitchStacks();
#define smx_SWITCH_TO_SSi()
#define smx_SWITCH_TO_TSi()

/* trigger smx_PendSV_Handler() */
#define smx_PENDSVH()            {*ARMM_NVIC_INT_CTRL = ARMM_FL_PENDSVSET; \
                                  sb_INT_ENABLE(); \
                                  __ISB(); }  /* <2> */

/* Macros to save the task suspend location (where it will resume).
   Called from ISR_ENTER() and SSR_ENTER(). For debug. */

#if defined(__ICCARM__)   /* IAR */
#if SMX_CFG_SAVE_SUSPLOC
#define smx_PROCESS_STACK 0x0004

#define smx_SAVE_SUSPLOC() \
   { \
      if (smx_ct->susploc == 0) \
      { \
         smx_ct->susploc = (void*) __get_LR(); \
      } \
   }

#define smx_SAVE_SUSPLOC_ISR() \
   { \
      if (smx_ct->susploc == 0) \
      { \
         u32* stackPointer; \
         if (__get_LR() & smx_PROCESS_STACK)         /* find EXC_RETURN's return stack */ \
            stackPointer = (u32*) __get_PSP();       /* read the Process Stack Pointer */ \
         else \
            stackPointer = (u32*) __get_MSP();       /* read the Main Stack Pointer */ \
         smx_ct->susploc = (void*) stackPointer[6];  /* return PC is the 6th item in the exception stack frame */ \
      } \
   }

#define smx_CLEAR_SUSPLOC() \
   { \
      smx_ct->susploc = 0; \
   }
#else
#define smx_SAVE_SUSPLOC()
#define smx_SAVE_SUSPLOC_ISR()
#define smx_CLEAR_SUSPLOC()
#endif

#elif defined(__GNUC__)   /* GNU */
#define smx_SAVE_SUSPLOC()
#define smx_SAVE_SUSPLOC_ISR()
#define smx_CLEAR_SUSPLOC()
#endif

#else
#error Define the porting macros for your compiler here in xarmm.h.

#endif  /* compiler checks */
/*==========================================================================*/

/*
Notes:

1. smx_ISR_EXIT(): The exception return does NOT reenable interrupts (clear
   BASEPRI or PRIMASK), so we need to do it before the return, in the
   case where we return to the point of interrupt. However, we don't want
   to allow interrupts at the tail of smx_ISR_EXIT() because they could invoke
   LSRs, and they would wait until the next interrupt or SSR, which could
   be a long time. We have already passed the check of smx_lqctr, and since
   an interrupt at the tail of smx_ISR_EXIT() would be nested, it would skip
   all the smx_ISR_EXIT code and return to the point of interrupt (due to the
   RETTOBASE check at the top). Fortunately, the processor DOES clear
   FAULTMASK on exception exit so we set it to 1, clear BASEPRI or PRIMASK,
   and then return. FAULTMASK inhibits all interrupts like PRIMASK and
   faults too. FAULTMASK only remains set for a few instructions, until
   the return, which clears it.

2. smx_PENDSVH(): A memory barrier instruction is needed after setting the
   PENDSV flag to ensure the exception happens immediately, not one or more
   instructions below, which could cause a problem, such as if GetCTRV()
   (which follows in smx_SSRExit()) is inlined. In this case, the return from
   exception would return in the middle of it, not at its first line, causing
   smx_ct->rv not to be actually read. The ENABLE() (or a function call if
   GetCTRV() were not inlined) seems to act as a memory barrier instruction,
   but we cannot rely on this and must use an explicit ISB.

3. Added DISABLE/ENABLE around read of ARMM_NVIC_INT_CTRL to make it atomic.
   If an ISR nests right after R0 is loaded with the address of this register
   but before reading it, then when the nested ISR returns, it appears
   RETTOBASE is not (yet?) set even though the original ISR is the outermost.
   In this case, it should have gone inside the if() and checked srnest,
   lq_ctr, and possibly run LSRs and scheduler, but instead, it returns to
   the point of interrupt, leaving any LSRs waiting and not clearing BR.
   This was detected in v5 with checks in the Idle task to see if BR is set
   and then analyzing the ETM trace. It's not clear why there should be a
   problem, but it was very repeatable, and always shows the nested ISR at
   the same place, right after loading R0 with ARMM_NVIC_INT_CTRL address.
   The problem was seen on Cortex-M7 and may or may not apply to others.
   Further testing seemed to show the problem is related to the MPU or our
   use of it in MPU-Plus, so this may not be a problem prior to v4.4.0, or
   if SMX_CFG_MPU is 0, but the change was applied to all versions to be safe.

4. In smx_MPU_BR_SAVE_ON/OFF(), (ARMM_NVIC_SYSHAN_CTRL_STATE & 0x7FF) != 0
   means an smx ISR interrupted a processor exception handler, such as SVC
   or PendSV. These are not smx ISRs, so we are currently in the outermost
   smx ISR. However, RETTOBASE is 0 because we are nested in one of these
   exceptions. SysTick is excluded because it is an smx ISR. Reserved bits
   are assumed to be future exceptions.

5. An ISR may need to reference a variable in the partition it is associated
   with, e.g. FS, USB, etc., and the task that happened to be interrupted is
   likely from a different partition and doesn't have the region in its MPA
   to access it. The variable cannot be put into sys_data because then the
   umode partition code would not be able to access it.
*/

#endif /* SMX_XARMM_H */

